<?php

namespace yifang\lib;

use InvalidArgumentException;
use yifang\utils\Str;
use yifang\App;

abstract class Manager
{
    /** @var App */
    protected $app;

    /**
     * 驱动
     * @var array
     */
    protected $drivers = [];

    /**
     * 驱动的命名空间
     * @var string
     */
    protected $namespace = null;

    public function __construct(App $app)
    {
        $this->app = $app;
    }

    /**
     * 获取驱动实例
     */
    protected function driver($name = null)
    {
        $name = $name ?: $this->getDefaultDriver();
        if (is_null($name)) {
            throw new InvalidArgumentException(sprintf(
                'Unable to resolve NULL driver for [%s].',
                static::class
            ));
        }
        return $this->drivers[$name] = $this->getDriver($name);
    }

    /**
     * 获取驱动实例
     */
    protected function getDriver($name)
    {
        return $this->drivers[$name] ?? $this->createDriver($name);
    }

    /**
     * 获取驱动类型
     */
    protected function resolveType($name)
    {
        return $name;
    }

    /**
     * 获取驱动配置
     */
    protected function resolveConfig($name)
    {
        return $name;
    }

    /**
     * 获取驱动类
     */
    protected function resolveClass($type)
    {
        if ($this->namespace || false !== strpos($type, '\\')) {
            $class = false !== strpos($type, '\\') ? $type : $this->namespace . Str::studly($type);
            if (class_exists($class)) {
                return $class;
            }
        }
        throw new InvalidArgumentException("Driver [$type] not supported.");
    }

    /**
     * 获取驱动参数
     */
    protected function resolveParams($name)
    {
        $config = $this->resolveConfig($name);
        return [$config];
    }

    /**
     * 创建驱动
     */
    protected function createDriver($name)
    {
        $type = $this->resolveType($name);
        $method = 'create' . Str::studly($type) . 'Driver';
        $params = $this->resolveParams($name);

        if (method_exists($this, $method)) {
            return $this->$method(...$params);
        }
        $class = $this->resolveClass($type);
        return $this->app->invokeClass($class, $params);
    }

    /**
     * 移除一个驱动实例
     */
    public function forgetDriver($name = null)
    {
        $name = $name ?? $this->getDefaultDriver();
        foreach ((array) $name as $cacheName) {
            if (isset($this->drivers[$cacheName])) {
                unset($this->drivers[$cacheName]);
            }
        }
        return $this;
    }

    /**
     * 默认驱动
     */
    abstract public function getDefaultDriver();

    /**
     * 动态调用
     */
    public function __call($method, $parameters)
    {
        return $this->driver()->$method(...$parameters);
    }
}
